
#ifndef _MODULAR_SYNTHESIS_CLASSES_
#define _MODULAR_SYNTHESIS_CLASSES_

#include <cstdlib>
#include <cstring>
#include <cmath>

class Module {
protected:
    static float samplerate;
    static long long time;

    float output;
    long long last_time;

public:
    static void SetSampleRate(float sr) { samplerate = sr; }
    static float GetSampleRate() { return samplerate; }
    static long long GetTime() { return time; }
    static void SetTime(long long t) { time = t; }
    static void IncTime() { time++; }

    Module() : output(0), last_time(time) {};
    virtual void Update() { };
    virtual float Out();
};



class Noise : public Module {
public:
    void Update();
};



class Mixer2 : public Module {
public:
    Module *input1;
    Module *input2;
    float gain1;
    float gain2;

    Mixer2() : gain1(1.0f), gain2(1.0f) {};
    void Update();
};


class Amplifier : public Module {
public:
    Module *input;
    Module *modulator;
    float gain;

    Amplifier() : input(NULL), modulator(NULL), gain(1.0f) {}
    void Update();
};


class SineOsc : public Module {
public:
    float phase;
    float frequency;
    Module *frequency_modulator;
    float modulation_amount;

    SineOsc() : phase(0), frequency(0), frequency_modulator(NULL), modulation_amount(0) {}
    void Update();
};


class LinearSlew : public Module {
protected:
    float delta;
    float target;
    bool target_reached;

public:
    LinearSlew() : delta(0) {};
    void SetTargetTime(float level, float delay);
    void SetTargetRate(float level, float rate);
    void Update();
};


class AREnvelope : public LinearSlew {
public:
    float attack;
    float release;
    bool retrigger;
    bool gate;
    bool release_stage;
    long long trigger_time;

    AREnvelope() : attack(0), release(0), retrigger(false), gate(false), release_stage(false), trigger_time(-1) {}
    void Update();
};


class Delay : public Module {
protected:
    float *buffer;
    int buffer_size;
    int write_pos;

public:
    Module *input;
    int delay_time;
    float feedback;
    float mix;

    Delay(float max_time);
    ~Delay();
    void SetTime(float t);
    void Update();
};


#endif
