
#ifndef _2OP_FM_SYNTHESIZER_CLASS_
#define _2OP_FM_SYNTHESIZER_CLASS_

#include <iostream>
#include <portaudio.h>
#include "module.h"

using namespace std;

class GUIControl {
    friend ostream &operator <<(ostream &, GUIControl &g);

protected:
    float *parameter;
    float min;
    float max;
    char inckey;
    char deckey;
    int size;
    char name[13];
    float delta;

public:
    GUIControl();
    GUIControl(const char *_n, float *_p, float _min, float _max, char _ik, char _dk, int s = 50);
    float GetDelta();
    float GetValue();
    void SetValue(float v);
    void AddValue(float v);
};

ostream &operator <<(ostream &, GUIControl &g);


class KeyCommand {
protected:
    GUIControl *control;
    float delta;

public:
    KeyCommand();
    void Set(GUIControl *_c = NULL, float d = 0);
    void Execute();
};


class Synth {
protected:
    // synth architecture
    SineOsc carrier;
    Amplifier carrier_amp;
    AREnvelope carrier_env;

    SineOsc modulator;
    Amplifier modulator_amp;
    AREnvelope modulator_env;
    SineOsc modulator_lfo;

    Delay delay;

    // portaudio stream
    PaStream *stream;

    // GUI
    KeyCommand keycommand[128];
    GUIControl control[20];
    int numcontrols;

public:
    Synth();
    ~Synth();

    void AddControl(const char *_n, float *_p, float _min, float _max, char _ik, char _dk, int s = 60);
    void ShowGUI();
    void PlayNote(int midinote);
    void Run();
};

#endif
